@@ -85,7 +85,7 @@ public:
   void collectLineTo(unsigned id, unsigned level, double x, double y);
   void collectArcTo(unsigned id, unsigned level, double x2, double y2, double bow);
   void collectNURBSTo(unsigned id, unsigned level, double x2, double y2, unsigned char xType, unsigned char yType, unsigned degree,
-                      std::vector<std::pair<double, double> > controlPoints, std::vector<double> knotVector, std::vector<double> weights);
+                      const std::vector<std::pair<double, double> > &ctrlPnts, const std::vector<double> &kntVec, const std::vector<double> &weights);
   void collectNURBSTo(unsigned id, unsigned level, double x2, double y2, double knot, double knotPrev, double weight, double weightPrev, unsigned dataID);
   void collectNURBSTo(unsigned id, unsigned level, double x2, double y2, double knot, double knotPrev, double weight, double weightPrev, const NURBSData &data);
   void collectPolylineTo(unsigned id, unsigned level, double x, double y, unsigned char xType, unsigned char yType,
@@ -141,6 +141,7 @@ public:
   void collectNameList(unsigned id, unsigned level);
   void collectName(unsigned id, unsigned level,  const WPXBinaryData &name, TextFormat format);
   void collectPageSheet(unsigned id, unsigned level);
+  void collectMisc(unsigned level, const VSDMisc &misc);
 
 
   // Style collectors
@@ -195,6 +196,7 @@ private:
 
   double _NURBSBasis(unsigned knot, unsigned degree, double point, const std::vector<double> &knotVector);
 
+  void _flushShape();
   void _flushCurrentPath();
   void _flushText();
   void _flushCurrentForeignData();
@@ -216,6 +218,17 @@ private:
   void appendCharacters(WPXString &text, const std::vector<unsigned char> &characters);
   void _convertDataToString(WPXString &result, const WPXBinaryData &data, TextFormat format);
   bool parseFormatId( const char *formatString, unsigned short &result );
+  void _appendField(WPXString &text);
+
+  // NURBS processing functions
+  bool _isUniform(const std::vector<double> &weights) const;
+  void _generatePolylineFromNURBS(unsigned degree, const std::vector<std::pair<double, double> > &controlPoints,
+                                  const std::vector<double> &knotVector, const std::vector<double> &weights);
+  void _generateBezierSegmentsFromNURBS(unsigned degree, const std::vector<std::pair<double, double> > &controlPoints,
+                                        const std::vector<double> &knotVector);
+  void _outputCubicBezierSegment(const std::vector<std::pair<double, double> > &points);
+  void _outputQuadraticBezierSegment(const std::vector<std::pair<double, double> > &points);
+  void _outputLinearBezierSegment(const std::vector<std::pair<double, double> > &points);
 
   bool m_isPageStarted;
   double m_pageWidth;
@@ -229,6 +242,7 @@ private:
   double m_originalY;
   XForm m_xform;
   XForm *m_txtxform;
+  VSDMisc m_misc;
   std::vector<WPXPropertyList> m_currentFillGeometry;
   std::vector<WPXPropertyList> m_currentLineGeometry;
   std::map<unsigned, XForm> *m_groupXForms;
@@ -248,15 +262,15 @@ private:
   std::map<unsigned short, VSDFont> m_fonts;
   unsigned m_currentLevel;
   bool m_isShapeStarted;
-  std::map<unsigned, unsigned> &m_groupMemberships;
   std::vector<std::map<unsigned, XForm> > &m_groupXFormsSequence;
   std::vector<std::map<unsigned, unsigned> > &m_groupMembershipsSequence;
+  std::vector<std::map<unsigned, unsigned> >::iterator m_groupMemberships;
   unsigned m_currentPageNumber;
   VSDOutputElementList *m_shapeOutputDrawing, *m_shapeOutputText;
   std::map<unsigned, VSDOutputElementList> m_pageOutputDrawing;
   std::map<unsigned, VSDOutputElementList> m_pageOutputText;
   std::vector<std::list<unsigned> > &m_documentPageShapeOrders;
-  std::list<unsigned> &m_pageShapeOrder;
+  std::vector<std::list<unsigned> >::iterator m_pageShapeOrder;
   bool m_isFirstGeometry;
 
   std::map<unsigned, NURBSData> m_NURBSData;
