@@ -168,6 +168,14 @@ void libvisio::VSDXMLParserBase::readGeometry(xmlTextReaderPtr reader)
       if (XML_READER_TYPE_ELEMENT == tokenType)
         readRelQuadBezTo(reader);
       break;
+    case XML_SPLINESTART:
+      if (XML_READER_TYPE_ELEMENT == tokenType)
+        readSplineStart(reader);
+      break;
+    case XML_SPLINEKNOT:
+      if (XML_READER_TYPE_ELEMENT == tokenType)
+        readSplineKnot(reader);
+      break;
     default:
       break;
     }
@@ -1010,6 +1018,7 @@ void libvisio::VSDXMLParserBase::readShape(xmlTextReaderPtr reader)
       m_shape.m_charList = tmpShape->m_charList;
       m_shape.m_paraList = tmpShape->m_paraList;
       m_shape.m_text = tmpShape->m_text;
+      m_shape.m_misc = tmpShape->m_misc;
     }
   }
 
@@ -1358,24 +1367,6 @@ void libvisio::VSDXMLParserBase::readCharIX(xmlTextReaderPtr reader)
       if (XML_READER_TYPE_ELEMENT == tokenType)
         readBoolData(doublestrikeout, reader);
       break;
-    case XML_RTLTEXT:
-      break;
-    case XML_USEVERTICAL:
-      break;
-    case XML_LETTERSPACE:
-      break;
-    case XML_COLORTRANS:
-      break;
-    case XML_ASIANFONT:
-      break;
-    case XML_COMPLEXSCRIPTFONT:
-      break;
-    case XML_LOCALIZEFONT:
-      break;
-    case XML_COMPLEXSCRIPTSIZE:
-      break;
-    case XML_LANGID:
-      break;
     default:
       break;
     }
@@ -1462,16 +1453,6 @@ void libvisio::VSDXMLParserBase::readParaIX(xmlTextReaderPtr reader)
       if (XML_READER_TYPE_ELEMENT == tokenType)
         ret = readByteData(align, reader);
       break;
-    case XML_BULLET:
-      break;
-    case XML_BULLETSTR:
-      break;
-    case XML_BULLETFONT:
-      break;
-    case XML_BULLETFONTSIZE:
-      break;
-    case XML_TEXTPOSAFTERBULLET:
-      break;
     case XML_FLAGS:
       if (XML_READER_TYPE_ELEMENT == tokenType)
         readUnsignedData(flags, reader);
@@ -1588,7 +1569,7 @@ void libvisio::VSDXMLParserBase::readSplineStart(xmlTextReaderPtr reader)
       break;
     }
   }
-  while (((XML_ELLIPTICALARCTO != tokenId && XML_ROW != tokenId) || XML_READER_TYPE_END_ELEMENT != tokenType) && 1 == ret);
+  while (((XML_SPLINESTART != tokenId && XML_ROW != tokenId) || XML_READER_TYPE_END_ELEMENT != tokenType) && 1 == ret);
   if (ret == 1)
     m_currentGeometryList->addSplineStart(ix, level, x, y, a, b, c, d);
 }
@@ -1666,7 +1647,7 @@ void libvisio::VSDXMLParserBase::readStencil(xmlTextReaderPtr reader)
 
 void libvisio::VSDXMLParserBase::readForeignData(xmlTextReaderPtr reader)
 {
-  VSD_DEBUG_MSG(("VSDXParser::readForeignData\n"));
+  VSD_DEBUG_MSG(("VSDXMLParser::readForeignData\n"));
   if (!m_shape.m_foreign)
     m_shape.m_foreign = new ForeignData();
 
@@ -1679,6 +1660,8 @@ void libvisio::VSDXMLParserBase::readForeignData(xmlTextReaderPtr reader)
       m_shape.m_foreign->type = 2;
     else if (xmlStrEqual(foreignTypeString, BAD_CAST("EnhMetaFile")))
       m_shape.m_foreign->type = 4;
+    else if (xmlStrEqual(foreignTypeString, BAD_CAST("MetaFile")))
+      m_shape.m_foreign->type = 0;
     xmlFree(foreignTypeString);
   }
   xmlChar *foreignFormatString = xmlTextReaderGetAttribute(reader, BAD_CAST("CompressionType"));
@@ -1713,6 +1696,8 @@ void libvisio::VSDXMLParserBase::_flushShape()
 
   m_collector->collectXFormData(m_currentShapeLevel+2, m_shape.m_xform);
 
+  m_collector->collectMisc(m_currentShapeLevel+2, m_shape.m_misc);
+
   if (m_shape.m_txtxform)
     m_collector->collectTxtXForm(m_currentShapeLevel+2, *(m_shape.m_txtxform));
 
@@ -1763,7 +1748,7 @@ void libvisio::VSDXMLParserBase::_flushShape()
     }
   }
 
-  if (m_shape.m_foreign)
+  if (m_shape.m_foreign && m_shape.m_foreign->data.size())
     m_collector->collectForeignData(m_currentShapeLevel+1, m_shape.m_foreign->data);
 
   if (!m_shape.m_fields.empty())
